--[[
  TOSHIBA REGZA用スクリプト

  2012.5.2
  ・DVDタイトルリストの表示に対応。
  ・DVDの音声が英語以外でも日本語字幕表示再生に対応。優先音声＋日本語字幕表示で再生します。

  REGZA Z1で動作テストしましたが、トランスコードが不安定です。
  ・再生後、BMSがリストを表示しなくなる事がある。
  ・REGZAの終了ボタンを押してTVに戻しても、音声は出力されるが、映像は出ない状態が1～2分続く。操作も受付けない。放っておけば正常に戻る。
  ・再度、REGZAからBMSを選択することはできるが、リストは表示されない。
  ・BMSの再起動が必要だが、BMSのQUITボタンではBMSが終了しないので、タスクマネージャで終了させなければならない。

]]


-- AVCをサポートする機種では true にしてください。
-- AVCをサポートしない機種では false にしてください。
AVCsupport = true

-- DVDのタイトルリストをタイトル番号、言語選択順に表示する場合は true にしてください
-- 言語選択、タイトル番号順に表示する場合は false にしてください
DVDSortTitleNo = false


-- この機種がサポートしているメディア情報のリスト
SUPPORT_MEDIA_LIST = {
  "image/jpeg:DLNA.ORG_PN=JPEG_SM",
  "image/jpeg:DLNA.ORG_PN=JPEG_MED",
  "audio/L16;rate=44100;channels=1:DLNA.ORG_PN=LPCM",
  "audio/L16;rate=44100;channels=2:DLNA.ORG_PN=LPCM",
  "audio/L16;rate=48000;channels=1:DLNA.ORG_PN=LPCM",
  "audio/L16;rate=48000;channels=2:DLNA.ORG_PN=LPCM",
  "audio/mpeg:DLNA.ORG_PN=MP3",
  "video/mpeg:DLNA.ORG_PN=MPEG_PS_NTSC",
  "video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_JP_T",
  "video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_JP_AAC_T",
  "video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_HD_60_AC3_T"
}


--[[
 GetMimeType 関数

 メディアファイルに応じたメディア情報を返す。

 引数：
   fname: ファイル名
   minfo: MediaInfo.dll などから得られた情報
]]
function GetMimeType(fname, minfo)
  
  if minfo.General.Format == "" then return "" end
  
  if minfo.General.Format == "NowRecording" then
    -- 録画中のファイルは MPEG-TS と仮定する。
    return "video/mpeg:DLNA.ORG_PN=MPEG_PS_NTSC;"
         .."DLNA.ORG_OP=11;DLNA.ORG_CI=0"
  end

  local ext = string.lower(string.match(fname, "%.(.-)$"))

  if ext == "jpg" and minfo.General.Format == "JPEG" then
    return "image/jpeg:DLNA.ORG_PN=JPEG_MED"
  end

  if ext == "mp3" and minfo.General.Format == "MPEG Audio" then
    return "audio/mpeg:DLNA.ORG_PN=MP3"
  end
  
  if ext == "wav" and minfo.General.Format == "Wave" then
    -- 正常に再生できないので無効とする。
    return ""
  end

  if minfo.Video.Format == "" then return "" end

  local fr = string.sub(minfo.Video.FrameRate, 1, 2)

  -- MPEG-PS
  if minfo.General.Format == "MPEG-PS" then
    return "video/mpeg:DLNA.ORG_PN=MPEG_PS_NTSC;"
         .."DLNA.ORG_OP=11;DLNA.ORG_CI=0"
  end
  
  -- MPEG-TS, BDAV (音声がAC-3,AACのみ再生)
  if minfo.General.Format == "MPEG-TS" or minfo.General.Format == "BDAV" then
    if minfo.Audio.Format == "AC-3" or minfo.Audio.Format == "AAC" then
      if minfo.Video.Format == "MPEG Video" then
        return "video/mpeg:DLNA.ORG_PN=MPEG_PS_NTSC;"
             .."DLNA.ORG_OP=11;DLNA.ORG_CI=0"
      elseif AVCsupport and minfo.Video.Format == "AVC"
       and  fr ~= "25" and fr ~= "50" and minfo.Video.Standard ~= "PAL" then
        return "video/mpeg:DLNA.ORG_PN=MPEG_PS_NTSC;"
             .."DLNA.ORG_OP=11;DLNA.ORG_CI=0"
      end
    end
  end

  -- トランスコード
  return "video/mpeg:DLNA.ORG_PN=MPEG_PS_NTSC;"
       .."DLNA.ORG_OP=10;DLNA.ORG_CI=0"
       ..":::TRANS:::"
end


--[[
 GetTranscodeCommand 関数
 
 トランスコードのコマンドを返す。

 引数：
   fname: ファイル名
   minfo: MediaInfo.dll などから得られた情報
]]
function GetTranscodeCommand(fname, minfo)

  local aspect = minfo.Video.DisplayAspectRatio
  if aspect == "" then aspect = "1.333" end
  --print("\r\nDEBUG: "..fname.." aspect="..aspect.."\r\n")
  local w = minfo.Video.Width
  local h = minfo.Video.Height

  -- この機種ではサイズによりアスペクト比を判定しているみたいなので
  -- mpeg2 のサイズとアスペクト比を固定にする。
  w = "720" h = "480"
  local vfs0 = "scale="..w..":"..h
  if aspect ~= "1.778" then
    local sw = w
    local sh = h
    if tonumber(aspect) < 1.778 then
      sw = math.floor(w / 1.778 * aspect + 0.5)
    else
      sh = math.floor(h * 1.778 / aspect + 0.5)
    end
    vfs0 = "scale="..sw..":"..sh..",expand="..w..":"..h
    aspect = "1.778"
  end

  local vfs = vfs0 .. ",pp=l5,harddup"


  -- DVDトランスコード用変数
  dvdopt = [[ -o "$_out_$" -oac lavc -ovc lavc -of mpeg -mpegopts format=dvd:tsaf
          -vf ]]..vfs..[[ -srate 48000 -af lavcresample=48000
          -ofps 30000/1001
          -channels 6
          -lavcopts vcodec=mpeg2video:vrc_buf_size=1835:vrc_maxrate=9800:vbitrate=5000:keyint=18:vstrict=0:acodec=ac3:abitrate=192:aspect=]]
          ..aspect
  local slangjalist = false
  local alangjalist = false
  local dvdres = ""

  -- DVDのトランスコード
  if minfo.General.Format == "ISO DVD" then
    dvdlength = {
      minfo.DVD.LENGTH1,minfo.DVD.LENGTH2,minfo.DVD.LENGTH3,minfo.DVD.LENGTH4,minfo.DVD.LENGTH5,
      minfo.DVD.LENGTH6,minfo.DVD.LENGTH7,minfo.DVD.LENGTH8,minfo.DVD.LENGTH9,minfo.DVD.LENGTH10,
      minfo.DVD.LENGTH11,minfo.DVD.LENGTH12,minfo.DVD.LENGTH13,minfo.DVD.LENGTH14,minfo.DVD.LENGTH15,
      minfo.DVD.LENGTH16,minfo.DVD.LENGTH17,minfo.DVD.LENGTH18,minfo.DVD.LENGTH19,minfo.DVD.LENGTH20,
      minfo.DVD.LENGTH21,minfo.DVD.LENGTH22,minfo.DVD.LENGTH23,minfo.DVD.LENGTH24,minfo.DVD.LENGTH25,
      minfo.DVD.LENGTH26,minfo.DVD.LENGTH27,minfo.DVD.LENGTH28,minfo.DVD.LENGTH29,minfo.DVD.LENGTH30,
      minfo.DVD.LENGTH31,minfo.DVD.LENGTH32,minfo.DVD.LENGTH33,minfo.DVD.LENGTH34,minfo.DVD.LENGTH35,
      minfo.DVD.LENGTH36,minfo.DVD.LENGTH37,minfo.DVD.LENGTH38,minfo.DVD.LENGTH39,minfo.DVD.LENGTH40,
      minfo.DVD.LENGTH41,minfo.DVD.LENGTH42,minfo.DVD.LENGTH43,minfo.DVD.LENGTH44,minfo.DVD.LENGTH45,
      minfo.DVD.LENGTH46,minfo.DVD.LENGTH47,minfo.DVD.LENGTH48,minfo.DVD.LENGTH49,minfo.DVD.LENGTH50,
      minfo.DVD.LENGTH51,minfo.DVD.LENGTH52,minfo.DVD.LENGTH53,minfo.DVD.LENGTH54,minfo.DVD.LENGTH55,
      minfo.DVD.LENGTH56,minfo.DVD.LENGTH57,minfo.DVD.LENGTH58,minfo.DVD.LENGTH59,minfo.DVD.LENGTH60,
      minfo.DVD.LENGTH61,minfo.DVD.LENGTH62,minfo.DVD.LENGTH63,minfo.DVD.LENGTH64,minfo.DVD.LENGTH65,
      minfo.DVD.LENGTH66,minfo.DVD.LENGTH67,minfo.DVD.LENGTH68,minfo.DVD.LENGTH69,minfo.DVD.LENGTH70,
      minfo.DVD.LENGTH71,minfo.DVD.LENGTH72,minfo.DVD.LENGTH73,minfo.DVD.LENGTH74,minfo.DVD.LENGTH75,
      minfo.DVD.LENGTH76,minfo.DVD.LENGTH77,minfo.DVD.LENGTH78,minfo.DVD.LENGTH79,minfo.DVD.LENGTH80,
      minfo.DVD.LENGTH81,minfo.DVD.LENGTH82,minfo.DVD.LENGTH83,minfo.DVD.LENGTH84,minfo.DVD.LENGTH85,
      minfo.DVD.LENGTH86,minfo.DVD.LENGTH87,minfo.DVD.LENGTH88,minfo.DVD.LENGTH89,minfo.DVD.LENGTH90,
      minfo.DVD.LENGTH91,minfo.DVD.LENGTH92,minfo.DVD.LENGTH93,minfo.DVD.LENGTH94,minfo.DVD.LENGTH95,
      minfo.DVD.LENGTH96,minfo.DVD.LENGTH97,minfo.DVD.LENGTH98,minfo.DVD.LENGTH99 
    }

    -- 日本語字幕の選択表示
    if minfo.DVD.SLANG.ja then
      if tonumber(minfo.DVD.ALANG.Count) > 1 or not minfo.DVD.ALANG.ja then
        slangjalist = true
      end
    end
    -- 日本語音声の選択表示
    if minfo.DVD.ALANG.ja and tonumber(minfo.DVD.ALANG.Count) > 1 then
      alangjalist = true
    end

    -- タイトルリスト
    if DVDSortTitleNo then
      dvdres = dvdTitleList1(slangjalist,alangjalist,tonumber(minfo.DVD.LONGEST))
    else
      dvdres = dvdTitleList2(slangjalist,alangjalist,tonumber(minfo.DVD.LONGEST))
    end

  return dvdres
  end


  -- 通常のトランスコード
  return [[

   [$_name_$ (TRANSCODE to MPEG-PS)]
   mencoder "$_in_$" -o "$_out_$"
   -oac lavc -ovc lavc -of mpeg -mpegopts format=dvd:tsaf
   -vf ]]..vfs..[[ -srate 48000 -af lavcresample=48000
   -ofps 30000/1001 -channels 6
   -lavcopts vcodec=mpeg2video:vrc_buf_size=1835:vrc_maxrate=9800:vbitrate=5000:keyint=18:vstrict=0:acodec=ac3:abitrate=192:aspect=]]
   ..aspect
end


--[[
*****************************************************
* DVDタイトルリストをタイトル番号、言語選択順に表示 *
*****************************************************
]]
function dvdTitleList1(slangjalist,alangjalist,longestNo)
  local longestdsp = ""
  local dvdtranslist = ""
  for i=1,99 do
    if dvdlength[i] ~= nil then
      -- 再生時間最長タイトルに ★を表示する
      if longestNo == i then longestdsp = " ★" else longestdsp = "" end
      -- 日本語字幕のタイトル
      if slangjalist then
        dvdtranslist =  dvdtranslist
          .."\n[Title:"..i..longestdsp.." (日本語字幕)]\n"
          ..[[mencoder -dvd-device "$_in_$" dvd://]]..i
          ..dvdopt
          .." -slang ja"
      end
      -- 日本語音声のタイトル
      if alangjalist then
        dvdtranslist =  dvdtranslist
          .. "\n[Title:"..i..longestdsp.." (日本語音声)]\n"
          ..[[mencoder -dvd-device "$_in_$" dvd://]]..i
          ..dvdopt
          .." -alang ja -slang ja -forcedsubsonly"
      end
      -- 字幕無しのタイトル
      if slangjalist then
        dvdtranslist =  dvdtranslist
          .."\n[Title:"..i..longestdsp.." (字幕無し)]\n"
          ..[[mencoder -dvd-device "$_in_$" dvd://]]..i
          ..dvdopt
          .. " -forcedsubsonly"
      end
      -- 音声、字幕の選択無しのタイトル
      if slangjalist == false and alangjalist == false then
        dvdtranslist =  dvdtranslist
          .."\n[Title:"..i..longestdsp.."]\n"
          ..[[mencoder -dvd-device "$_in_$" dvd://]]..i
          ..dvdopt
          .. " -forcedsubsonly"
      end
    end
  end
  return dvdtranslist
end

--[[
*****************************************************
* DVDタイトルリストを言語選択、タイトル番号順に表示 *
*****************************************************
]]
function dvdTitleList2(slangjalist,alangjalist,longestNo)
  local longestdsp = ""
  local dvdtranslist = ""

  -- 日本語字幕のタイトル
  if slangjalist then
    for i=1,99 do
      if dvdlength[i] ~= nil then
        -- 再生時間最長タイトルに ★を表示する
        if longestNo == i then longestdsp = " ★" else longestdsp = ""  end
        dvdtranslist =  dvdtranslist
          .."\n[(日本語字幕) Title:"..i..longestdsp.."]\n"
          ..[[mencoder -dvd-device "$_in_$" dvd://]]..i
          ..dvdopt
          .." -slang ja"
      end
    end
  end

  -- 日本語音声のタイトル
  if alangjalist then
    for i=1,99 do
      if dvdlength[i] ~= nil then
        -- 再生時間最長タイトルに ★を表示する
        if longestNo == i then longestdsp = " ★" else longestdsp = ""  end
        dvdtranslist =  dvdtranslist
          .. "\n[(日本語音声) Title:"..i..longestdsp.."]\n"
          ..[[mencoder -dvd-device "$_in_$" dvd://]]..i
          ..dvdopt
          .." -alang ja -slang ja -forcedsubsonly"
      end
    end
  end

  -- 字幕無しのタイトル
  if slangjalist then
    for i=1,99 do
      if dvdlength[i] ~= nil then
        -- 再生時間最長タイトルに ★を表示する
        if longestNo == i then longestdsp = " ★" else longestdsp = ""  end
        dvdtranslist =  dvdtranslist
          .."\n[(字幕無し)Title:"..i..longestdsp.."]\n"
          ..[[mencoder -dvd-device "$_in_$" dvd://]]..i
          ..dvdopt
          .. " -forcedsubsonly"
      end
    end
  end

  -- 音声、字幕の選択無しのタイトル
  if slangjalist == false and alangjalist == false then
    for i=1,99 do
      if dvdlength[i] ~= nil then
        -- 再生時間最長タイトルに ★を表示する
        if longestNo == i then longestdsp = " ★" else longestdsp = ""  end
        dvdtranslist =  dvdtranslist
          .."\n[Title:"..i..longestdsp.."]\n"
          ..[[mencoder -dvd-device "$_in_$" dvd://]]..i
          ..dvdopt
          .. " -forcedsubsonly"
      end
    end
  end

  return dvdtranslist
end
