--[[
  共通スクリプト
]]


function OrdSet(t)
  local s = {}
  for i,v in ipairs(t) do s[v] = i end
  return s
end


--[[
 GetScriptFileName 関数

 スクリプト名を決定する。

 引数：
   in_header: クライアントが送出した HTTP ヘッダの内容。
   ip_addr: クライアントの IP アドレス。
   in_uri: クライアントが問い合わせてきた URI アドレス。
   
 戻り値：
   スクリプト名。""（空白）を返すとそのクライアントにはサービスを提供しない。
]]
function GetScriptFileName(in_header, ip_addr, in_uri)

  --print("\r\nDEBUG:", ip_addr, "\r\n")
  
--[[ 特定の IP アドレスにはサービスを提供しないようにする例
  if ip_addr == "192.168.0.13" then
    return ""
  end
-- ]]

  local i1 = string.find(in_header, "X-AV-Client-Info:", 1, true)

  local s = ""
  if i1 then
    s = string.match(in_header, 'mn="(.-)"', i1)
    --print("\r\nDEBUG:"..s.."\r\n")
  end

  if s and string.sub(s, 1, 6) == "BRAVIA" then

    local BRAVIA_KDL_JP11 = OrdSet({
     "HX920", "HX820", "HX720", "NX720", "EX72S", "EX720", "EX420", "CX400"
    })

    local s1 = string.match(s, "BRAVIA KDL%-%d+(%w+)")
    if s1 and BRAVIA_KDL_JP11[s1] then
      -- MP4 対応ブラビア
      return "BRAVIA-KDL-JP11"
    -- regexpr.matches 関数では Java の正規表現をほぼそのまま使えます。
    elseif regexpr.matches(s, "BRAVIA KDL-[0-9]{2}(F5|W5|ZX5|[EHNLC]X[0-9]{2}[0SR])") then
      -- AVCHD 対応ブラビア
      return "BRAVIA-KDL-JP5"
    else --if regexpr.matches(s, "BRAVIA KDL-[0-9]{2}([A-Z]{1,2}1|J5|V5)") then
      return "BRAVIA-KDL-JP1"
    end

--[[ Lua の正規表現だけを使った場合の例
    local BRAVIA_KDL_JP1 = OrdSet({"F1", "F2", "V1", "J1", "M1", "V5", "J5"})
    local BRAVIA_KDL_JP5 = OrdSet({"F5", "W5", "ZX5", "EX", "HX", "NX", "LX"})

    local s1, s2 = string.match(s, "BRAVIA KDL%-%d+(%u+)(%d+)")
    if s1 and s2 then
      if BRAVIA_KDL_JP1[s1..s2] then
        return "BRAVIA-KDL-JP1"
      elseif BRAVIA_KDL_JP5[s1..s2] or BRAVIA_KDL_JP5[s1] then
        return "BRAVIA-KDL-JP5"
      end
    end
-- ]]

  end

  i1 = string.find(in_header, "X-AV-Physical-Unit-Info:", 1, true)
  s = nil
  if i1 then
    s = string.match(in_header, 'pa="(.-)"', i1)
  end

  if s then
    if regexpr.matches(s, "Sony KDL-[0-9]{2}[JXWV][357][0-9]{3}") then
      -- Sony Bravia 3000/5000/7000 Series（未テスト）
      return "BRAVIA-KDL-JPU"
    end
  end

  i1 = string.find(string.upper(in_header), "USER-AGENT:", 1, true)
  s = nil
  if i1 then
    s = string.match(in_header, "(Panasonic Digital Media Player/1%.00)", i1)
    if s then
      -- Panasonic ビエラ・ワンセグ SV-ME970
      return "VIERA-SV-ME970"
    end
  -- TOSHIBA REGZA用
    s = string.match(in_header, "UPnP/1.0 DLNADOC/1.50 Intel_SDK_for_UPnP_devices/1.2", i1)
    if s then
      return "REGZA"
    end
  end



  -- IP アドレスと同じ名前のスクリプトファイル（例：192.168.0.17.lua）があるならそれを使う。
  if ScriptFileExists(ip_addr) then
    return ip_addr
  end

--[[ 特定の URI 問い合わせ時までスクリプト名の決定を留保する例。
-- 通常は、一番最初の問い合わせである in_uri = "/desc.xml" の際にスクリプト名が決定されるが、
-- もう少し後の問い合わせのヘッダ内容によりスクリプト名を決定したほうがよい場合などに。
  if in_uri == "/upnp/control/connection_manager" then
    if s == "Super DLNA TV" then
      return "DLNA_TV3"
    end
    return "NONAME"
  else
    return "" -- 決定を留保
  end
-- ]]

  return "NONAME"
end

